// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: https://svn/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Algorithms/ITK/include/mapITKPointSetToPointSetMetricControlBase.h $
*/




#ifndef __ITK_POINTSET_TO_POINTSET_METRIC_CONTROL_BASE_H
#define __ITK_POINTSET_TO_POINTSET_METRIC_CONTROL_BASE_H

#include "mapITKPointSetToPointSetMetricControlInterface.h"

namespace map
{
	namespace algorithm
	{
		namespace itk
		{

			/*! @class ITKPointSetToPointSetMetricControlBase
			This class serves as base class for all
			template specialization of itk::PointSetToPointSetMetrics.
			It realizes some basic metric handling, to reduce the code in the control classes
			@remark ITKPointSetToPointSetMetricControlBase instantiates a concrete metric when constructed.
			You can set a new one, but ITKPointSetToPointSetMetricControlBase has always an optimizer under control.
			@template TConcreteMetric type of the concrete metric controlled by the class
			@ingroup ITK
			@ingroup MetricControl
			*/
			template<class TConcreteMetric>
			class ITKPointSetToPointSetMetricControlBase : public
				ITKPointSetToPointSetMetricControlInterface<typename TConcreteMetric::MovingPointSetType, typename TConcreteMetric::FixedPointSetType>
			{
			public:
				typedef ITKPointSetToPointSetMetricControlBase<TConcreteMetric> Self;
				typedef ITKPointSetToPointSetMetricControlInterface<typename TConcreteMetric::MovingPointSetType, typename TConcreteMetric::FixedPointSetType>
				Superclass;
				typedef ::itk::SmartPointer<Self>         Pointer;
				typedef ::itk::SmartPointer<const Self>   ConstPointer;

				itkTypeMacro(ITKPointSetToPointSetMetricControlBase, ITKPointSetToPointSetMetricControlInterface);

				typedef TConcreteMetric ConcreteMetricType;
				typedef typename ConcreteMetricType::Pointer ConcreteMetricPointer;
				typedef typename Superclass::MetricBaseType MetricBaseType;
				typedef typename Superclass::MVMetricBaseType MVMetricBaseType;
				typedef typename Superclass::PointSetToPointSetMetricBaseType PointSetToPointSetMetricBaseType;

				/*! setter for the optimizer
				@eguarantee no fail
				@param pMetric the ConcreteMetricType object that has to be set. Must not be NULL (otherwise an exception will be thrown).
				*/
				virtual void setMetric(ConcreteMetricType* pMetric);

				virtual MetricBaseType* getMetric();
				virtual const MetricBaseType* getMetric() const;

				virtual MVMetricBaseType* getMVMetric();
				virtual const MVMetricBaseType* getMVMetric() const;

				virtual PointSetToPointSetMetricBaseType* getPointSetToPointSetMetric();
				virtual const PointSetToPointSetMetricBaseType* getPointSetToPointSetMetric() const;

				virtual ConcreteMetricType* getConcreteMetric();
				virtual const ConcreteMetricType* getConcreteMetric() const;

				/*! The modification time (MT) of an metric controler is the the maximum of his MT
				 * and the MT of the controlled metric.  */
				virtual unsigned long GetMTime() const;

			protected:
				virtual ~ITKPointSetToPointSetMetricControlBase();
				ITKPointSetToPointSetMetricControlBase();

				ConcreteMetricPointer _spMetric;

			private:
				//No copy constructor allowed
				ITKPointSetToPointSetMetricControlBase(const Self& source);
				void operator=(const Self&);  //purposely not implemented
			};
		}
	}
}

#ifndef MatchPoint_MANUAL_TPP
#include "mapITKPointSetToPointSetMetricControlBase.tpp"
#endif

#endif
